"""


Author: 
    Inspyre Softworks

Project:
    Inspyre-Toolbox

File: 
    inspyre_toolbox/chrono/__init__.py
 

Description:
    

"""
from inspyre_toolbox.chrono.sleep import sleep


__all__ = ['sleep']
