"""
The software organization information for the project.
"""
from inspyre_toolbox.common.about.author.dev_team import DEVELOPER_TEAM


SOFTWARE_ORG = {
        'name': 'Inspyre-Softworks',
        'developers': DEVELOPER_TEAM,
        'urls': {
                'main':   'https://inspyre.tech',
                'github': 'https://github.com/Inspyre-Softworks',
                'gitlab': 'https://gitlab.com/inspyre-softworks',
                'pypi':   'https://pypi.org/user/tblackstone/',
                }
        }
""" The software organization information for the project. """


SOFTWARE_ORG_URLS = SOFTWARE_ORG['urls']
""" The software organization URLs for the project. """


SOFTWARE_ORG_NAME = SOFTWARE_ORG['name']
""" The software organization name for the project. """
