
LEAD_DEVELOPER = {
        'name': 'Taylor-Jayde Blackstone',
        'email': '<t.blackstone@inspyre.tech>',
        'urls': {
                'github': 'https://github.com/tayjaybabee',
                'gitlab': 'https://gitlab.com/tayjaybabee',
        }
    }




DEVELOPER_TEAM = {
        'lead': LEAD_DEVELOPER,
        }


__all__ = [
    'DEVELOPER_TEAM',
    ]
