"""


Author: 
    Inspyre Softworks

Project:
    Inspyre-Toolbox

File: 
    inspyre_toolbox/type_man/comparators/string.py
 

Description:
    

"""


def case_insensitive_compare(string1: str, string2: str) -> bool:
    """
    Compare two strings in a case-insensitive manner.

    Parameters:

        string1 (str):
            The first string to compare.

        string2 (str):
            The second string to compare.

    Returns:
        bool: True if the strings are equal, False otherwise.

    Since:
        v1.6.0

    Example Usage:
        >>> from ins
